from django.urls import reverse

import pytest

from ..tables import ProjectIndexTable


@pytest.mark.django_db
class TestAssociationRelativeActions:
    def test_actions(self, project):
        table = ProjectIndexTable([project])
        actions_cell = table.rows[0].get_cell('actions')
        assert 'update-link' in actions_cell
        assert (
            reverse(
                'association:project:update',
                args=(project.association.pk, project.pk),
            )
            in actions_cell
        )
        assert 'delete-link' in actions_cell
        assert (
            reverse(
                'association:project:delete',
                args=(project.association.pk, project.pk),
            )
            in actions_cell
        )

    def test_without_update_or_delete(self, project):
        class TestTable(ProjectIndexTable):
            actions_update_url = None

        table = TestTable([project])
        actions_cell = table.rows[0].get_cell('actions')
        assert 'update-link' not in actions_cell
        assert 'delete-link' in actions_cell

        class TestTable(ProjectIndexTable):
            actions_delete_url = None

        table = TestTable([project])
        actions_cell = table.rows[0].get_cell('actions')
        assert 'update-link' in actions_cell
        assert 'delete-link' not in actions_cell
