from itertools import groupby

from django.core.management.base import BaseCommand

from benevalibre.base.views import AdminOnlyStatsPage


class Command(BaseCommand):
    help = 'Dump some usage statistics'

    def output(self, s):
        self.stdout.write(self.style.SUCCESS(s))

    def handle(self, *args, **options):
        page_context = AdminOnlyStatsPage().get_page_context()
        msg_stats = (
            "L'application Bénévalibre compte :\n"
            "- {nbAsso} associations,\n"
            "- {nbUser} utilisateurs,\n"
            "- {nbBenevalo} actes de bénévolat saisis, "
            "pour une durée totale de {totalDuration} "
            "et {totalDistance:.2f} km."
        ).format(**page_context)
        self.output(msg_stats)
        self.output("\nDomaines d'activité des assos :\n")

        def regroup_key(activities):
            return activities['activity_field__activity_field_group__name']

        for group, details in groupby(
            page_context['activitiesData'], regroup_key
        ):
            details = list(details)  # no way to rewind grouper
            total = sum(detail['total_association'] for detail in details)
            self.output('- {} : {}'.format(total, group))
            for activity in details:
                self.output(
                    '  - {total_association} : {activity_field__name}'.format(
                        **activity
                    )
                )
