from django.db import models


class AbstractTaxonomy(models.Model):
    class Meta:
        abstract = True

    name = models.CharField("nom", max_length=255)
    description = models.TextField("description", blank=True, null=True)

    def __str__(self):
        return self.name


class RoleManager(models.Manager):
    def get_benevole(self):
        # /!\ il peut y en avoir plusieurs et ça engendrera
        # l'exception attendue
        return self.get(new_comers=True)

    def get_dirigeant(self):
        # /!\ il peut y en avoir zéro ou plusieurs et ça engendrera
        # l'exception attendue
        return self.get(manage_association=True)

    def delegables_by(self, role):
        # Un rôle peut déléguer à tout ses « sous-rôles ».
        # Un sous rôle ne peut pas gagner des permission.
        # Donc si une permission du rôle est True, celle du sous rôle
        # peut être n'importe.
        # Si une permission du rôle est False, celle du sous rôle
        # doit être False.
        permissions = [
            r.name
            for r in role._meta.fields
            if isinstance(r, models.BooleanField) and r.name != 'new_comers'
        ]
        incapacities = {k: False for k in permissions if not getattr(role, k)}

        return self.filter(**incapacities)


class AbstractRole(AbstractTaxonomy):
    class Meta:
        abstract = True

    objects = RoleManager()

    new_comers = models.BooleanField(
        "rôle par défaut dans l'association", default=False
    )

    # aspect animateur
    list_users = models.BooleanField(
        "liste les utilisat⋅eurs⋅rices", default=False
    )
    delegates = models.BooleanField("délègue ses permissions", default=False)
    manage_benevalos = models.BooleanField("gère le bénévolat", default=False)
    manage_levels = models.BooleanField("gère les niveaux", default=False)
    manage_projects = models.BooleanField("gère les projets", default=False)
    manage_categories = models.BooleanField(
        "gère les catégories", default=False
    )

    # aspects dirigeant
    manage_roles = models.BooleanField("gère les rôles", default=False)
    manage_association = models.BooleanField(
        "gère l'association", default=False
    )

    def __str__(self):
        return self.name
